"""
Battle the robot hand to hand
slow robots
minimal jump - no jump jets
duck
punch
the enemy robot does shoot you, dodge by jumping or ducking
if you can beat him to half health, he fires missiles at you
    which destroys your robot

You get out of the wreckage and hop in a car
You are racing away from the giant robot, whos feet keep
    coming down
Heading for the emp that has been set up
You get to a bridge over a deep mine, which he blows up sending 
    the car tumbling down
He crosses through the emp field (You have to activate it
    at the right time), you also get a chance to grab a better
    gun from the rack.
Without controls, he tumbles over the cliff into the shaft
Unfortunately, the disturbance causes you to slip over the egde
    as well as the ground gives way
    
You are both falling down the shaft, and he manages to power
    up his backup generators
You land in a hanging mine cart that starts sliding away

His robot is pretty busted, but he still manages to crawl after you
He chases you through the mine shaft with you firing away
The mine opens into the woods

You take the mining bombs you got from the cart and decide
to finish this sucker once and for all

Have to climb up his legs and get the bomb planted in his face,
    and detonate it
    
Finally, the robot explodes and the enemy comes out.  You still
have to face him one on one.  He can jump into the arms of 
his former bot and shoot their lasers at you.
At some point you manage to cut his hand off.  If you can 
pick up his hand, you can also control the robot arms.

This triggers the final cut scene where you laser him to death
"""

import pyglet,random,math
pyglet.resource.path = ["data"]
pyglet.resource.reindex()
from pyglet.window import key

def qsprite(path,anchor=(0,0)):
    """Quickly load an image and make a sprite for it"""
    img = pyglet.resource.image(path)
    img.anchor_x,img.anchor_y = anchor
    return pyglet.sprite.Sprite(img)

logo = qsprite("logo.png")

tt = pyglet.text.HTMLLabel('<b>Showdown</b>')
tt.color = (255,255,255,0)
tt.alpha = 0
tt.alpha_delay = 100
tt.y = 50

he = pyglet.text.HTMLLabel('<b>Hit Enter to Begin</b>')
he.color = (255,255,255,0)
he.alpha = 0
he.alpha_delay = 200
he.x,he.y = [100,50]
texts = [tt,he]

class State:
    def __init__(self,win):
        self.win = win
        self.setup()
    def setup(self):
        pass
    def on_draw(self):
        self.win.clear()
        pyglet.gl.glScalef(2,2,2)
        self.realdraw()
        pyglet.gl.glScalef(0.5,0.5,0.5)
    def realdraw(self):
        logo.draw()
        [x.draw() for x in texts]
    def update(self,dt,x):
        for title in texts:
            title.alpha_delay-=1
            if title.alpha_delay<=0:
                title.alpha_delay = 0
                title.alpha+=10
                if title.alpha>255: 
                    title.alpha=255
                title.color = (255,255,255,title.alpha)
    def on_key_press(self,symbol,mods):
        if symbol==pyglet.window.key.RETURN:
            self.win.state = Game(self.win)


"""    s = math.sin(-x.par.rotation*(math.pi/180.0))
    c = math.cos(-x.par.rotation*(math.pi/180.0))
    newxoff = x.off[0]*c-x.off[1]*s
    newyoff = x.off[0]*s+x.off[1]*c
    return newxoff,newyoff"""
    
def rotate(p1,p2,angle):
    """Rotate p1 about p2, return rotated p1"""
    s = math.sin(angle*(math.pi/180.0))
    c = math.cos(angle*(math.pi/180.0))
    x = p1[0]-p2[0]
    y = p1[1]-p2[1]
    xx = x*c-y*s
    yy = x*s+y*c
    xx+=p2[0]
    yy+=p2[1]
    return xx,yy
            
def col(point,part):
    radius = (part.width+part.height)/2
    center = rotate([part.width//2,part.height//2],[part.off[0],part.off[1]],-part.rotation)
    if math.sqrt((point[0]-part.x)**2+(point[1]-part.y)**2)<radius:
        return True
    return False
            
class robot:
    def __init__(self,x,y):
        self.x = x
        self.y = y
        self.dir = dir
        
        self.r1_lbod = qsprite("r1_lbod.png",anchor=(15,0))
        self.r1_lbod.x = x
        self.r1_lbod.y = y
        self.r1_lbod.off = (0,0)
        
        self.r1_bod = qsprite("r1_bod.png",anchor=(22,0))
        self.r1_bod.par = self.r1_lbod
        self.r1_bod.off = (0,50)
        
        self.r1_arm1 = qsprite("r1_arm1.png",anchor=(12,12))
        self.r1_arm1.rotation = 90
        self.r1_arm1.dir = 1
        self.r1_arm1.par = self.r1_bod
        self.r1_arm1.off = (0,10)
        
        self.r1_arm11 = qsprite("r1_arm1.png",anchor=(12,12))
        self.r1_arm11.rotation = 0
        self.r1_arm11.dir = 1
        self.r1_arm11.par = self.r1_arm1
        self.r1_arm11.off = (50,0)

        self.r1_arm2 = qsprite("r1_arm1.png",anchor=(12,12))
        self.r1_arm2.rotation = 90
        self.r1_arm2.dir = -1
        self.r1_arm2.par = self.r1_bod
        self.r1_arm2.off = (0,15)
        
        self.r1_foot = qsprite("r1_foot.png",anchor=(47,52))
        self.r1_foot.par = self.r1_lbod
        self.r1_foot.off = (0,0)
        self.width = 60
        
        self.parts = [self.r1_lbod,self.r1_bod,self.r1_arm1,self.r1_arm11,self.r1_arm2,self.r1_foot]
        for x in self.parts:
            x.rot = x.rotation
            x.oi = x.image
        self.flipped = 0
    def flip(self):
        self.flipped = 1-self.flipped
        for x in self.parts:
            ax = x.image.anchor_x
            ay = x.image.anchor_y
            x.image = x.oi.get_transform(flip_x=self.flipped)
            x.image.anchor_x = x.width-ax
            x.image.anchor_y = ay
            x.off = (-x.off[0],x.off[1])
    def draw(self):
        self.r1_arm2.draw()
        self.r1_lbod.draw()
        self.r1_bod.draw()
        self.r1_foot.draw()
        self.r1_arm11.draw()
        self.r1_arm1.draw()
    def update(self):
        dx = 0
        if key.A in self.hold:
            #~ if not self.flipped:
                #~ self.flip()
            dx -=5
        if key.D in self.hold:
            #~ if self.flipped:
                #~ self.flip()
            dx += 5
        self.r1_lbod.x+=dx
        if dx>0:
            if self.r1_lbod.x+self.width>self.other.x and self.r1_lbod.x<self.other.x+self.other.width:
                self.r1_lbod.x = self.other.x-self.width
        if dx<0:
            if self.r1_lbod.x<self.other.x+self.other.width and self.r1_lbod.x+self.width>self.other.x:
                self.r1_lbod.x = self.other.x+self.other.width
        for a in [self.r1_arm1,self.r1_arm2]:
            if dx and a.rot>45:
                a.rot += a.dir
                if a.rot>120:
                    a.dir=-1
                if a.rot<60:
                    a.dir=1
            else:
                a.rot += (90-a.rot)/10.0
                
        self.x = self.r1_lbod.x
        self.y = self.r1_lbod.y
        
        for x in self.parts:
            if self.flipped:
                x.rotation = 360-x.rot
            else:
                x.rotation = x.rot
            if not getattr(x,"par",None): continue
            newxoff,newyoff = rotate([x.off[0],x.off[1]],[0,0],-x.par.rotation)
            x.x = x.par.x+newxoff
            x.y = x.par.y+newyoff

class Game(State):
    def setup(self):
        self.can_hold = [key.A,key.D]
        self.hold = []
        self.r1 = robot(50,70)
        self.r1.hold = self.hold
        self.r2 = robot(200,70)
        self.r2.flip()
        self.r2.hold = []
        self.r1.other = self.r2
        self.r2.other = self.r1
        self.ground = qsprite("ground.png")
        self.bg = qsprite("bg1.png")
        self.island = qsprite("island.png")
        self.cam = [0,0]
    def realdraw(self):
        self.bg.draw()
        pyglet.gl.glTranslatef(-self.cam[0]/2,-self.cam[1],0)
        amt = 450
        c = self.cam[0]/2
        self.island.x = (c//amt)*amt
        self.island.draw()
        self.island.x = (c//amt+1)*amt
        self.island.draw()
        self.island.x = (c//amt+2)*amt
        self.island.draw()
        pyglet.gl.glTranslatef(self.cam[0]/2,self.cam[1],0)
        pyglet.gl.glTranslatef(-self.cam[0],-self.cam[1],0)
        self.ground.x = (self.cam[0]//320)*320
        self.ground.draw()
        self.ground.x = (self.cam[0]//320+1)*320
        self.ground.draw()
        self.ground.x = (self.cam[0]//320+2)*320
        self.ground.draw()
        self.r1.draw()
        self.r2.draw()
        pyglet.gl.glTranslatef(self.cam[0],self.cam[1],0)
    def update(self,dt,x):
        self.r1.update()
        self.r2.update()
        d = (self.cam[0]-(self.r1.x-110))/25.0
        self.cam[0]-=d
        hand = list(rotate(self.r1.r1_arm11.off,[0,0],-self.r1.r1_arm11.rotation))
        hand[0]+=self.r1.r1_arm11.x
        hand[1]+=self.r1.r1_arm11.y
        if self.r1.r1_arm1.rot<10:
            for x in self.r2.parts:
                if col(hand,x):
                    self.r2.r1_lbod.x+=5
    def on_key_press(self,symbol,mods):
        if symbol in self.can_hold and not symbol in self.hold:
            self.hold.append(symbol)
        if symbol==key.SPACE and self.r1.r1_arm1.rot>45:
            self.r1.r1_arm1.rot = 0
    def on_key_release(self,symbol,mods):
        if symbol in self.can_hold and symbol in self.hold:
            self.hold.remove(symbol)

class GameWindow(pyglet.window.Window):
    def __init__(self,*args,**kwargs):
        pyglet.window.Window.__init__(self,*args,**kwargs)
        self.state = State(self)
        pyglet.clock.schedule(self.update, 1/60.0)
    def on_draw(self):
        self.state.on_draw()
    def update(self,*args):
        self.state.update(*args)
    def on_key_press(self,*args):
        self.state.on_key_press(*args)
    def on_key_release(self,*args):
        self.state.on_key_release(*args)

def main():
    main_win = GameWindow(width=640,height=480)

    pyglet.app.run()